﻿#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma version=2.40
#pragma IgorVersion=7
#pragma moduleName=archull

// --------------------- Project Updater header ----------------------
// If you have installed or updated this project using the IgorExchange 
// Projects Installer (http: www.igorexchange.com/project/Updater) you 
// can be notified when new versions are released.
static constant kProjectID=7399 // the project node on IgorExchange
static strconstant ksShortTitle="ArcHull" // the project short title on IgorExchange

// https://www.wavemetrics.com/user/tony

// This package adds a collection of algorithmically-defined baselines
// for spectral data. It's named for the arc hull type baseline, in which
// a circular arc with adjustable depth is added to the spectrum (ie.,
// the spectrum is bent), and the lower portion of a convex hull is
// calculated for the resultant spectrum. The baseline consists of the
// sum of the arc and the convex hull.

// How to use the Arc Hull package

// Arc Hull requires that a spectrum is plotted in the top graph.
// Selecting Macros - Arc Hull will add a panel on the left side of the
// graph. You can choose from a popup menu which of the spectra in the
// graph to use for the calculation. A working copy of the baseline and
// baseline-subtracted spectrum are appended to the plot; these are
// updated as you adjust the Arc Hull parameters.

// Adjust arc depth to find an optimal value. The arc depth SetVar
// increments by 10% of its value. Setting arc depth to zero creates a
// convex hull (a.k.a. 'rubber band') type baseline.

// Spline: baseline is a cubic spline using the convex hull as nodes.
// When arc depth is non-zero the nodes are more like a subset of
// alphashape vertices. Adjust Smooth and Depth parameters to produce an
// optimal number of nodes.

// Iterative: the function selected in the Fit Func popup is fitted
// iteratively. Any fit point with a negative residual (positive if the
// negative peaks checkbox is selected) is replaced by the corresponding
// point from the input data after each iteration, thus eliminating
// points with a positive residual, so ultimately eliminating peaks from
// the fit. The fit output is used as the input for the next iteration.
// Lieber C.A. and Mahadevan-Jansen A. (2003) Applied Spectroscopy 57:
// 1363-1367.

// Smooth: the baseline is calculated for a smoothed copy of the
// spectrum, then subtracted from the original. With the right choice of
// smoothing factor, the baseline will pass through spectral noise rather
// than underlying the noise. Note that the baseline-subtracted output is
// not smoothed with respect to the original data, the smoothing is
// applied only to a temporary copy that's discarded after calculating
// the baseline.
   
// Base level: the output (baseline-subtracted) spectrum is offset from
// zero by this amount. Useful for negative peaks that extend down from a
// non-zero value. For convex baselines, use negative arc depth (you'll
// have to type a negative value in the setvar).

// Negative Peaks checkbox: subtract the top part of the convex hull.
// Useful for dealing with downward facing peaks, or for when you have
// a reverse-scaled vertical axis.

// Use graph limits checkbox: calculate baseline within the horizontal
// axis limits of the graph. Unless the entire wave can be seen in the
// graph, the baseline-subtracted output wave will have a smaller range
// than the data wave.

// Subtract: creates a copy of the baseline and a baseline-subtracted
// output spectrum in the current data folder and appends them to the plot.

// All in one: creates a baseline-subtracted output spectrum for each
// wave on the graph (other than waves with _BL and _sub suffixes)
// using the current settings for arc depth, smoothing, base level and
// checkbox values.

// Closing the Arc Hull panel will remove temporary package-created waves
// from the graph and clean up after the package, as will initializing on
// another graph window.

// For X-Y data, the X wave needs to be monatonic for Arc Hull calculation.

// Execute archull#demo() to demo.

// Version history
// 2.40 3 June 2021: End of the line for this project. Development will be
// continued as part of the Baselines project:
// https://www.wavemetrics.com/project/Baselines
// 2.30 10 July 2020 Bug fix, adds missing /Z flag for an optional wave 
// parameter (I omitted to do this when switching rtGlobals pragma 
// setting for a previous release)
// 2.11 9 Jan 2020 Adds an option (kRecolorTraces=1) where only the
// selected trace, the baseline, and the subtraction preview are shown in
// colors other than grey.
// 2.10 works with any axes
// 2.09 fixes for error if an incompatible wave is selected for processing
// 2.07 Bug fix: graph hook was updating fit needlessly.
// 2.06 Record fit coefficients for iterative fit in wave note
// 2.05 Made most functions static
// 2.04 Added headers for Project Updater.
// 2.03 11/11/17 Should be backward compatible for Igor 6.
// 2.02 decreased panel size for compatibility with smaller or lower
// resolution screens. And a bug fix.
// 2.0 Introduces iterative fitting of CurveFit functions with
// elimination of either positive or negative residual points
// Versions 1.3, 1.31, 1.32 Add option to create baseline within x-range
// of graph and spline type baseline using convex hull as nodes
// Version 1.2, 14/4/17 Adds smoothing and base level options

// If "use graph limits" is selected, the baseline will update as the
// plot area changes. You may want to disable this so that you can zoom
// in or out and see the fit in a different context, then reenter a fit
// parameter to update the fit. Set kUpdateOnZoom=0 to prevent this
// behaviour:
static constant kUpdateOnZoom=1
static constant kRecolorTraces=1 // set to 0 to disable trace color changes
static constant kAppendBaselines=0 // whether to append baseline after subtraction
static constant kRemoveOriginal=0 // choose whether to remove data trace after baseline subtraction

static strconstant ksBaselineSuffix="_BL"
static strconstant ksSubtractedSuffix="_sub"

Menu "Analysis"
	"Arc Hull", /Q, archull#Initialise()
end

//Menu "Macros"
//	"Arc Hull",/Q, archull#Initialise()
//end

static function Initialise()
	
	if(goodbye())
		return 0
	endif
	
	if (strlen(WinList("*",";","WIN:1"))==0)  // no graphs
		DoAlert 0, "Arc hull baseline requires a trace plotted in a graph window."
		return 0
	endif
	
	// clear any package detritus from the last used arc hull graph
	clearGraph()
	
	// initialize (or reinitialize) the package data folder
	CreatePackageFolder()
	
	// make control panel and append package waves to graph
	variable success
	success = makePanel()
	if (kRecolorTraces==1)
		// save current trace colors and set all to grey
		resetColors(getGraph(), 3)
	endif
	if (success) // we have a trace selected
		updatePlot()
	endif
end

static function/DF GetDFREF()
	DFREF dfr = root:Packages:ArcHull
	if (DataFolderRefStatus(dfr) != 1)
		DFREF dfr = CreatePackageFolder()
	endif
	return dfr
end

static function /DF CreatePackageFolder()
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:ArcHull
	DFREF dfr = root:Packages:ArcHull
	return dfr
end

static function makePanel()

	// make sure the top graph is visible
	string graphStr=WinName(0,1)
	DoWindow /F $graphStr
	
	// Create a data folder reference variable
	DFREF dfr = GetDFREF()
	
   // make sure there's space to the left of the graph for the panel
	GetWindow /Z $graphStr wsize
	if (v_left<190)
		#if IgorVersion() >= 7
			MoveWindow /W=$graphStr 190, v_top, -1, -1
		#else
			MoveWindow /W=$graphStr 190, v_top, 190+(v_right-v_left), v_bottom
		#endif
	endif
			
	// make panel
	NewPanel /K=1/N=CCH_panel/W=(190,0,0,355)/HOST=$graphStr/EXT=1 as "Arc Hull Controls"
	ModifyPanel /W=$graphStr#CCH_panel, noEdit=1
	
	string traceStr=StringFromList(0, getTraces())
	wave /Z w=TraceNameToWaveRef(graphStr, traceStr)
	variable depth=(WaveMax(w)-WaveMin(w))*.15
	
	variable i=0, deltaY=25, groupw=180, font=12
	v_left=25
	v_top=5
	
	GroupBox group0,pos={v_left-20,v_top+deltaY*i},size={groupw,deltaY*2},title="Data wave"
	GroupBox group0,fSize=font
	i+=1
	// store values internally in these controls
	PopupMenu popTrace, mode=1, Value=archull#getTraces(), title="",pos={v_left,v_top+deltaY*i},size={130,20}
	PopupMenu popTrace, help={"select data wave" }, proc=archull#popupTrace, fSize=font
	i+=1.5
	GroupBox group1 pos={v_left-20,v_top+deltaY*i},size={groupw,deltaY*9},title="Baseline parameters"
	GroupBox group1 fSize=font
	i+=1
	CheckBox check_CCH, pos={v_left,v_top+deltaY*i}, fSize=font, value=1, proc=archull#checkboxFunc, title="Arc hull"
	CheckBox check_CCH, mode=1,help={"Arc hull type baseline"}
	i+=1
	CheckBox check_spline, pos={v_left,v_top+deltaY*i}, fSize=font, value=0, proc=archull#checkboxFunc, title="Hull spline"
	CheckBox check_spline, mode=1,help={"cubic spline through nodes from convex hull"}
	i+=1
	CheckBox check_itFit, pos={v_left,v_top+deltaY*i}, fSize=font, value=0, proc=archull#checkboxFunc, title="Iterative"
	CheckBox check_itFit, mode=1,help={"iterative fit with elimination"}
	i+=1
	SetVariable setvarDepth,pos={v_left+10,v_top+deltaY*i},size={130,16},title="Arc depth", focusring=0
	SetVariable setvarDepth,limits={-Inf,Inf,abs(depth/10)},value=_NUM:depth, bodyWidth=70
	SetVariable setvarDepth,help={"depth of arc"}, fSize=font, proc=archull#setvarFunc
	
	PopupMenu popFitFunc, mode=2, title="Fit Func",pos={v_left,v_top+deltaY*i},size={130,20}
	PopupMenu popFitFunc, help={"select fit function" }, proc=archull#popupFitFunction, fSize=font, disable = 1
	PopupMenu popFitFunc, Value="line;poly 3;poly 4;gauss;lor;exp;dblexp;sin;hillequation;sigmoid;power;lognormal;"
	i+=1
	SetVariable setvarSmooth,pos={v_left+10,v_top+deltaY*i},size={130,16},title="Smooth"
	SetVariable setvarSmooth,limits={0,32767,1},value=_NUM:20, bodyWidth=70, focusring=0
	SetVariable setvarSmooth,help={"binomial smoothing factor"}, fSize=font, proc=archull#setvarFunc
	i+=1
	SetVariable setvarOffset,pos={v_left+10,v_top+deltaY*i},size={130,16},title="Base level"
	SetVariable setvarOffset,limits={-Inf,Inf,1},value=_NUM:0, bodyWidth=70, focusring=0
	SetVariable setvarOffset,help={"Offset from zero"}, fSize=font, proc=archull#setvarFunc
	i+=1
	CheckBox check_neg, pos={v_left,v_top+deltaY*i}, fSize=font, value=0, proc=archull#checkboxFunc, title="Negative peaks"
	CheckBox check_neg, help={"Fit baseline to top of convex hull"}
	i+=1
	CheckBox check_win, pos={v_left,v_top+deltaY*i}, fSize=font, value=0, proc=archull#checkboxFunc, title="Use graph limits"
	CheckBox check_win, help={"Calculate baseline within limits of graph X-axis"}
	i+=1.5
	Button CCH_sub,pos={60,v_top+deltaY*i},size={70,20},title="Subtract", proc=archull#buttonFunc
	Button CCH_sub,help={"Subtract baseline and save a copy"}, fSize=font
	i+=1
	Button CCH_all,pos={60,v_top+deltaY*i},size={70,20},title="All in one", proc=archull#buttonFunc
	Button CCH_all,help={"Subtract baseline from all traces"}, fSize=font
	
	if (strlen(traceStr)==0)
		return 0
	endif
	return 1
end

// checks for change in range of x-axis
static function hookXrange(STRUCT WMWinHookStruct &s)
			
	if ( (kUpdateOnZoom==0) || (s.eventCode!=8) )
		return 0
	endif
	
	string graphStr = getGraph()
	string panelStr = graphStr + "#CCH_panel"
	ControlInfo /W=$panelStr check_win
	if (V_Value == 0)
		return 0
	endif
	
	GetAxis /Q bottom
	Make /free/N=2 freeLimits = {v_min, v_max}
	
	DFREF dfr = GetDFREF()
	wave /Z w_limits = dfr:w_limits
	if (WaveExists(w_limits) == 0)
		Duplicate /O freeLimits dfr:w_limits /Wave=w_limits
	endif
	
	w_limits -= freeLimits
	w_limits = abs(w_limits)
	if (sum(w_limits) > 1e-12)
		doFit()
	endif
	w_limits = freeLimits
	
	return 0
end

static function selectTrace(string graphStr, string traceStr)
	if (kRecolorTraces == 1)
		graphStr = selectstring(strlen(graphStr)==0, graphStr, getGraph())
		resetColors(graphStr, 1) // set all to grey
		if (strlen(traceStr))
			restoreTraceColor(graphStr, traceStr)
		endif
		return 1
	endif
	return 0
end

// update baseline and ensure package waves are displayed
static function updatePlot([string trace])
	
	DFREF dfr = GetDFREF()
	
	string graphStr = getGraph()
	string panelStr = graphStr + "#CCH_panel"
				
	RemoveFromGraph /W=$graphStr/Z w_base, w_sub, w_nodesY
	
	if (paramisdefault(trace))
		ControlInfo /W=$panelStr popTrace
		trace = s_value
	endif
	
	wave /Z w_data = traceNameToWaveRef(graphStr, trace)
	if (waveexists(w_data) == 0)
		return 0
	endif
	
	selectTrace(graphStr, trace)	
	// do fit to create baseline
	variable success = doFit(trace=trace)
	if (success == 0) // could not fit selected data wave
		return 0
	endif
	
	wave /SDFR=dfr w_base, w_sub
	wave /Z w_dataX = XWaveRefFromTrace(graphStr, trace)
		
	if (WaveExists(w_dataX))
		wave w_baseX = dfr:w_baseX
		appendToSameAxes(graphStr, trace, w_base, w_baseX, offset=1, unique=1)
		appendToSameAxes(graphStr, trace, w_sub, w_baseX, unique=1)
	else
		appendToSameAxes(graphStr, trace, w_base, $"{}", offset=1, unique=1)
		appendToSameAxes(graphStr, trace, w_sub, $"{}", unique=1)
	endif
		
	wave /Z/SDFR=dfr w_nodesX, w_nodesY
	ControlInfo /W=$panelStr check_spline
	if (v_value)
		appendToSameAxes(graphStr, trace, w_nodesY, w_nodesX, offset=1, unique=1)
		ModifyGraph /W=$graphStr mode(w_nodesY)=3, marker(w_nodesY)=42
	else
		RemoveFromGraph /W=$graphStr/Z w_nodesY
	endif
	SetWindow $graphStr hook(hArcHull)=archull#hookXrange
	return 1
end

static function checkboxFunc(STRUCT WMCheckboxAction &s)
		
	if (s.eventCode == -1)
		return 0
	endif
	
	string graphStr = ParseFilePath(0, s.win, "#", 0, 0)
	
	if (stringmatch(s.ctrlName, "check_CCH"))
		CheckBox check_spline win=$s.win, value=0
		CheckBox check_ItFit win=$s.win, value=0
		SetVariable setvarDepth win=$s.win, disable=0
		PopupMenu popFitFunc win=$s.win, disable=1
		
		ControlInfo /W=$s.win setvarDepth
		if (v_value == 0)
			// set default depth
			ControlInfo /W=$s.win popTrace
			wave /Z w = TraceNameToWaveRef(graphStr, S_Value)
			if (WaveExists(w)) // just in case
				variable depth = (WaveMax(w) - WaveMin(w)) * 0.15
				SetVariable setvarDepth, win=$s.win,limits={-Inf,Inf,abs(depth/10)},value=_NUM:depth
			endif
		endif
	endif
		
	if (stringmatch(s.ctrlName, "check_spline"))
		CheckBox check_CCH win=$s.win, value=0
		CheckBox check_ItFit win=$s.win, value=0
		PopupMenu popFitFunc win=$s.win, disable=1
		SetVariable setvarDepth win=$s.win, disable=0
	endif
	
	if (stringmatch(s.ctrlName, "check_ItFit"))
		CheckBox check_CCH win=$s.win, value=0
		CheckBox check_spline win=$s.win, value=0
		PopupMenu popFitFunc win=$s.win, disable=0
		SetVariable setvarDepth win=$s.win, disable=1
	endif
	
	updatePlot()
	
	return 0
end

static function setvarFunc(STRUCT WMSetVariableAction &s)
	
	switch (s.eventCode)
		case 1:
		case 2:
 		case 8:
 				
		if (cmpstr(s.ctrlName, "setvarDepth")==0)
			if (s.dval == 0)			
				string graphStr = ParseFilePath(0, s.win, "#", 0, 0)
				ControlInfo /W=$s.win popTrace
				wave /Z w = TraceNameToWaveRef(graphStr, s_value)
				if (WaveExists(w) == 0)
					return 0
				endif
				SetVariable setvarDepth win=$s.win,limits={-Inf,Inf,(WaveMax(w)-WaveMin(w))*.15}
			else
				// reset increment value to 10% of current value
				SetVariable setvarDepth win=$s.win,limits={-Inf,Inf,abs(s.dval/10)}
			endif
		endif
		
		if (cmpstr(s.ctrlName, "setvarSmooth")==0)
			if (mod(s.dval, 1))
				s.dval = round(s.dval)
				SetVariable setvarSmooth, win=$s.win, value=_NUM:s.dval
			endif
		endif
		
		doFit()
		
		
	endswitch
	return 0
end

static function popupFitFunction(STRUCT WMPopupAction &s)
	switch (s.eventCode)
		case 3: // hover
			if (igorVersion() >= 9)
				doFit(fName=s.popStr)
			endif
			break
		case 2: // select
		case 4: // dismiss, Igor 9
			doFit()		
	endswitch
	return 0
end

static function popupTrace(STRUCT WMPopupAction &s)	
	
	string graphStr = ParseFilePath(0, s.win, "#", 0, 0)	
	switch (s.eventCode)
		case -1 :
			RemoveFromGraph /W=$graphStr/Z w_base, w_sub, w_nodesY
			SetWindow $graphStr hook(hArcHull)=$""
			if (kRecolorTraces == 1)
				resetColors(graphStr, 0)
			endif
			break
		case 3 : // hover
			if (igorVersion() >= 9)
				selectTrace(graphStr, s.popStr)
			endif
			break
		case 2 : // select
			// set default depth each time a trace is selected
			ControlInfo /W=$s.win check_CCH
			if (V_Value)
				wave /Z w = TraceNameToWaveRef(graphStr, s.popStr)
				if (WaveExists(w) == 0)
					return 0
				endif
				variable depth = (WaveMax(w) - WaveMin(w)) * 0.15
				SetVariable setvarDepth, win=$s.win,limits={-Inf,Inf,abs(depth/10)},value=_NUM:depth
			endif
		case 4 : // dismiss (and select)	
			updatePlot(trace=s.popStr) // does fit and ensures package waves are displayed
			break
	endswitch	
	return 0
end

static function resetDepth([string trace])
	
	trace = selectstring (paramisdefault(trace), trace, getSelectedTrace())
	string strGraph = getGraph()
	string strPanel = strGraph + "#CCH_panel"
	ControlInfo /W=$strPanel check_CCH
	if (V_Value)
		wave /Z w = TraceNameToWaveRef(strGraph, trace)
		if (WaveExists(w) == 0)
			return 0
		endif
		variable depth = (WaveMax(w) - WaveMin(w)) * 0.1
		SetVariable setvarDepth, win=$strPanel,limits={-Inf,Inf,abs(depth/10)},value=_NUM:depth
	endif
end

static function /T getSelectedTrace()
	ControlInfo /W=$getGraph() + "#CCH_panel" popTrace
	return s_value	
end

static function buttonFunc(STRUCT WMButtonAction &s)
	
	if (s.eventCode!=2)
		return 0
	endif
	
	string graphStr = ParseFilePath(0, s.win, "#", 0, 0)
	string traceStr = ""
		
	strswitch(s.ctrlName)
		case "CCH_sub":
			subtractBaseline(getSelectedTrace(), 0)
			break
		case "CCH_all":
			string msg = "subtract baseline from all traces using current settings?\r"
			msg += "existing baselines will be overwritten!"
			DoAlert 1, msg
			if (v_flag == 2)
				return 0
			endif
					
//			string listStr = getTraces()
//			PopupMenu popTrace, win=$s.win, Value=archull#getTraces()
//			variable i
//			for(i=0;i<ItemsInList(listStr); i+=1)
//				PopupMenu popTrace, win=$s.win, mode=i+1
//				doFit()
//				subtractBaseline(1)
//			endfor
			
			string listStr = getTraces()
			variable i
			for(i=0;i<ItemsInList(listStr); i+=1)
				traceStr = stringfromlist(i, listStr)
				updatePlot(trace=traceStr)
				subtractBaseline(traceStr, 1)
			endfor
						
			break
	endswitch

	return 0
end

static function /S getTraces()
	
	string graphStr = getGraph()
	string listStr = TraceNameList(graphStr,";",1+4)
	string removeStr = "w_nodesY;"
	removeStr += ListMatch(listStr, "w_base")
	removeStr += ListMatch(listStr, "*" + ksSubtractedSuffix)
	removeStr += ListMatch(listStr, "*" + ksBaselineSuffix)
	
	// remove traces with insufficient number of points or no data
	variable i
	for(i=0;i<ItemsInList(listStr);i+=1)
		wave w = TraceNameToWaveRef(graphStr, StringFromList(i, listStr))
		if (numpnts(w)<5 || numtype(WaveMax(w))!=0)
			removeStr += StringFromList(i, listStr) + ";"
		endif
	endfor
	
	listStr = RemoveFromList(removeStr, listStr, ";", 0)
	return listStr
end

static function doFit([string trace, string fName])
	
	string graphStr = getGraph()
	string panelStr = graphStr + "#CCH_panel"
	
	if (paramisdefault(trace))
		ControlInfo /W=$panelStr popTrace
		trace = s_value
	endif
	
	DFREF dfr = GetDFREF()
	
	KillWaves /Z dfr:w_baseX // won't die if it's used on plot....
	
	variable depth, varSm, varOffset, negPeaks, GraphLim
	ControlInfo /W=$panelStr setvarDepth
	depth = v_value
	ControlInfo /W=$panelStr setvarSmooth
	varSm = v_value
	ControlInfo /W=$panelStr setvarOffset
	varOffset = v_value
	ControlInfo /W=$panelStr check_neg
	negPeaks = v_value
	ControlInfo /W=$panelStr check_win
	GraphLim = v_value
	
	wave /Z w = TraceNameToWaveRef(graphStr, trace)
	if (WaveExists(w) == 0)
		return 0
	endif
	wave /Z w_x = XWaveRefFromTrace(graphStr, trace)
	
	if (numtype(depth) != 0) // some problem with the selected data wave
		return 0
	endif
	
	Make /free/N=2 w_pRange={0,numpnts(w)-1}, w_Xrange=NaN
	variable p1 = 0, p2 = numpnts(w) - 1
		
	if (GraphLim)
		string xAxisName = StringByKey("XAXIS", TraceInfo(graphStr,trace,0))
		GetAxis /W=$graphStr/Q $xAxisName
		w_Xrange = {v_min,V_max}
		Sort w_Xrange, w_Xrange
		if (WaveExists(w_x))
			if (w_x[1] < w_x[0])
				Sort /R w_Xrange, w_Xrange
			endif
			FindLevel /Q/P w_x, w_Xrange[0]
			if (v_flag == 0)
				w_pRange[0] = V_LevelX
			endif
			FindLevel /Q/P w_x, w_Xrange[1]
			if (v_flag == 0)
				w_pRange[1] = V_LevelX
			endif
		else
			w_pRange[0] = x2pnt(w, w_Xrange[0])
			w_pRange[1] = x2pnt(w, w_Xrange[1])
		endif
	endif
	Sort w_pRange, w_pRange
	p1 = max(0, w_pRange[0])
	p2 = min(p2, w_pRange[1])
	
	// create waves to display baseline and baseline subtraction
	Duplicate /O/R=[p1,p2] w dfr:w_sub /WAVE=w_sub
	Duplicate /O/R=[p1,p2] w dfr:w_base /WAVE=w_base
	if (WaveExists(w_x))
		Duplicate /O/R=[p1,p2] w_x dfr:w_baseX /WAVE=w_x
	endif
	
	string strNote = ""
	
	if (numpnts(w) < 3)
		w_sub = NaN
		w_base = NaN
		return 0
	endif
	
	ControlInfo /W=$panelStr check_ItFit
	if (v_value)
		Duplicate /O w_sub dfr:w_ItFit /WAVE=w_ItFit
		if (varSm)
			Smooth varSm, w_ItFit
		endif
		Duplicate /free w_sub w_next
		variable i, conv = Inf, success = 0
		
		if (paramisdefault(fName))
			ControlInfo /W=$panelStr popFitFunc
			fName = s_value
		endif
		
		for (i=0;i<1e4;i+=1)
			// fit w_ItFit with fName, output to w_base
			success = FitWrap(w_ItFit, w_x, w_base, fName)
			if (success == 0)
				break
			endif
			w_next = (w_ItFit > w_base) ? w_base : w_ItFit
			if (negPeaks)
				w_next = (w_ItFit < w_base) ? w_base : w_ItFit
			endif
			if (success && i<10) // make at least 10 iterations
				w_ItFit = w_next
				continue
			endif
			// convergence test - get a count of the number of points changed in this iteration
			w_ItFit = 1 / (w_next - w_ItFit)
			WaveStats /Q/M=1 w_ItFit
			if (i>10 && v_npnts<=conv) // converged
				break
			else
				conv = v_npnts
				w_ItFit = w_next
			endif
		endfor
		w_sub = w[p1+p] - w_base + varOffset
		sprintf strNote, "type=iterative;function=%s;iterations=%d;errors=%d;smoothing=%g;", fName, i+1, !success, varSm
	else // do arc hull calculation
		if (varSm>0)
			Smooth varSm, w_sub
		endif
		// make a copy of the (possibly smoothed) data wave
		Duplicate /free w_sub w_ref
		// calculate arc hull based on (possibly smoothed) data
		ArcHull(w_sub, depth, w_x=w_x, top=negPeaks)
		w_base = w_ref - w_sub
		w_sub = w[p1+p] - w_base + varOffset
		sprintf strNote, "type=ArcHull;depth=%g;smoothing=%g;", depth, varSm
	endif
	
	ControlInfo /W=$panelStr check_spline
	if (v_value)
		
		
		wave w_XHull, w_YHull
		duplicate /O w_XHull dfr:w_nodesX /wave=w_nodesX
		duplicate /O w_YHull dfr:w_nodesY /wave=w_nodesY
		
		if (depth != 0)
			w_nodesY = getY(w_nodesX, w_ref, w_x)
		endif
	
		if (numpnts(w_nodesX) > 3)
			if (WaveExists(w_x))
				Interpolate2 /T=2/E=2/Y=w_base/X=w_x/I=3 w_nodesX, w_nodesY
			else
				Interpolate2 /T=2/E=2/Y=w_base/I=3 w_nodesX, w_nodesY
			endif
			w_sub = w[p1+p] - w_base + varOffset
		endif
		strNote = replaceStringByKey("type", strNote, "HullSpline", "=")
	endif
	
	if (GraphLim)
		if (WaveExists(w_x))
			v_min = w_x[0]
			v_max = w_x[numpnts(w_x) - 1]
		endif
		sprintf strNote, "%sXmin=%g;Xmax=%g;", strNote, v_min, v_max
	endif
	note /K w_base "Baseline Parameters:" + strNote
	
	return 1
end

// status 0: reset to saved colors
// status 1: set all to grey
// status 2: save current
// status 3: save current and set all to grey
static function resetColors(string graphStr, variable status)
	
	DFREF dfr = GetDFREF()
	string traceStr
	variable i, numTraces
		
	if (status == 0)
		wave /Z/T w_traces = dfr:w_traces
		wave /Z w_colors = dfr:w_colors
		if (WaveExists(w_colors) == 0)
			return 0
		endif
		numTraces=numpnts(w_traces)
		for(i=0;i<numTraces;i+=1)
			ModifyGraph /W=$graphStr/Z rgb($w_traces[i])=(w_colors[i][0],w_colors[i][1],w_colors[i][2])
		endfor
		KillWaves /Z w_colors, w_traces
		return 1
	endif
	
	string traceListStr = TraceNameList(graphStr, ";", 1+4)
	numTraces = ItemsInList(traceListStr)
	
	if (status&2)
		Make /O/N=0/T dfr:w_traces /wave=w_traces
		Make /O/N=(0,3) dfr:w_colors /wave=w_colors
	else
		wave /Z/T w_traces = dfr:w_traces
		wave /Z w_colors = dfr:w_colors
	endif
		
	for(i=0;i<numTraces;i+=1)
		traceStr = StringFromList(i,traceListStr)
		if (status&2) // save current
			w_traces[i] = {traceStr}
			wave w_rgb = getColor(graphStr, traceStr)
			w_colors[i][] = {{w_rgb[0]},{w_rgb[1]},{w_rgb[2]}}
		else // check to see if a trace has been added since the list was created
			if (WaveExists(w_traces))
				FindValue /TEXT=traceStr/TXOP=4/Z w_traces
				if (V_Value == -1)
					w_traces[numpnts(w_traces)]={traceStr}
					wave w_rgb = GetColor(graphStr, traceStr)
					w_colors[DimSize(w_colors, 0)][] = {{w_rgb[0]},{w_rgb[1]},{w_rgb[2]}}
				endif
			endif
		endif
		if (status & 1) // set to grey
			ModifyGraph /W=$graphStr/Z rgb($traceStr)=(56797,56797,56797)
		endif
	endfor
end

// use this with kRecolorTraces=1 to restore the saved color of a single trace
static function restoreTraceColor(string graphStr, string traceStr)
		
	DFREF dfr = GetDFREF()
	wave /T w_traces = dfr:w_traces
	wave w_colors = dfr:w_colors
	
	FindValue /TEXT=traceStr/Z w_traces
	if (V_value > -1)
		make /free w = {56797,56797,56797}
		w = abs(w - w_colors[V_Value][q])
		if (sum(w) < 1000)
			ModifyGraph /W=$graphStr/Z rgb($traceStr)=(39321,1,31457)
		else
			ModifyGraph /W=$graphStr/Z rgb($traceStr)=(w_colors[V_value][0],w_colors[V_value][1],w_colors[V_value][2])
		endif
	else
		ModifyGraph /W=$graphStr/Z rgb($traceStr)=(39321,1,31457)
	endif
end

static function getY(v_x, w, w_x)
	variable v_x
	wave /Z w, w_x
	
	if (WaveExists(w_x))
		FindLevel /Q /P w_x, v_x
		if (v_flag == 0)
			return w[V_LevelX]
		endif
	else
		return w(v_x)
	endif
	return NaN
end

// subtracts arc hull baseline from w (w is overwritten)
function ArcHull(w, depth [, w_x, top])
	wave /Z w, w_x
	variable depth, top
	
	variable useXwave=0
	if (ParamIsDefault(w_x)==0 && WaveExists(w_x))
		useXwave=1
	endif
	
	top = ParamIsDefault(top) ? 0 : top
	// top=1 for top hull
	
	// add concave function
	variable radius
	if (useXwave)
		radius = (WaveMax(w_x) - WaveMin(w_x)) / 2
		w += depth * (w_x[p] - w_x[0])^2 / radius^2
	else
		variable x1 = leftx(w), x2 = pnt2x(w, numpnts(w) - 1)
		radius = abs(x2 - x1) / 2
		w += depth * (x-x1)^2 / radius^2
	endif
	
	Duplicate /free w w_base
	
	if (useXwave == 0)
		Duplicate /free w w_x
		// redimension to make sure we don't get hull vertices
		// outside the range of w_x owing to difference in precision
		// of output from ConvexHull
		Redimension /D w_x
		w_x = x
	endif
	
	ConvexHull /Z w_x, w
	wave w_XHull, w_YHull

	if (top == 0)
		Reverse /P w_XHull, w_YHull
	endif
	// negative depth will subtract top part of convex hull
	// an offset will likely need to be applied
	
	WaveStats /q w_XHull
	Rotate -v_minloc, w_XHull, w_YHull
	SetScale /p x, 0, 1, w_XHull, w_YHull
	WaveStats /q w_XHull
	DeletePoints v_maxloc+1, numpnts(w_XHull)-v_maxloc-1, w_XHull, w_YHull
	
	if (useXwave)
		Interpolate2 /T=1/Y=w_base/X=w_x/I=3 W_XHull, W_YHull
	else
		Interpolate2 /T=1/Y=w_base/I=3 W_XHull, W_YHull
	endif

	w -= w_base
end

// a wrapper for built-in fit functions
static function FitWrap(w, w_x, w_out, fName)
	wave /Z w, w_x, w_out
	string fName
		
	DebuggerOptions
	variable sav_debug=V_debugOnError
	DebuggerOptions debugOnError=0 // switch this off in case the fit fails
	
	variable V_FitError=0, V_fitOptions=4
	try
		strswitch (fName)
			case "line":
				CurveFit /Q/N line, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "poly 3":
				CurveFit /Q/N poly 3, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "poly 4":
				CurveFit /Q/N poly 4, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "gauss":
				CurveFit /Q/N Gauss, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "lor":
				CurveFit /Q/N lor, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "exp":
				CurveFit /Q/N exp, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "dblexp":
				CurveFit /Q/N dblexp, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "sin":
				CurveFit /Q/N sin, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "hillequation":
				CurveFit /Q/N hillequation, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "sigmoid":
				CurveFit /Q/N sigmoid, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "power":
				CurveFit /Q/N power, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
			case "lognormal":
				CurveFit /Q/N lognormal, w /D=w_out /X=w_x /NWOK; AbortOnRTE
				break
		endswitch
	catch
		if (V_AbortCode == -4)
			Print "Error during curve fit:"
			variable CFerror = GetRTError(1)	// 1 to clear the error
			Print GetErrMessage(CFerror)
		endif
		return 0
	endtry
	
	DebuggerOptions debugOnError=sav_debug
	
	if (V_FitError == 0) // record fit coefficients in baseline wavenote
		wave w_coef = w_coef
		string coefStr
		variable i
		sprintf coefstr, "w_coef={"
		for(i=0;i<numpnts(w_coef);i+=1)
			sprintf coefstr, "%s%g,", coefstr, w_coef[i]
		endfor
		coefstr = RemoveEnding(coefstr, ",")+"};"
		note /K w_out, coefstr
	endif
	
	return (V_FitError	==0)
end

static function subtractBaseline(string traceStr, variable overwrite)
	
	string graphStr = getGraph()
	string panelStr = graphStr + "#CCH_Panel"
	
	wave /Z w = TraceNameToWaveRef(graphStr, traceStr)
	
	if (WaveExists(w) == 0)
		return 0
	endif
	
	wave /Z w_dataX = XWaveRefFromTrace(graphStr, traceStr)
	
	DFREF CCHdfr = GetDFREF()
	wave /Z/SDFR=CCHdfr w_sub, w_base, w_baseX
	
	DFREF CurrentDfr = $GetDataFolder(1)
		
	// save a copy of the baseline
	string strNewName = CleanupName(NameOfWave(w) + ksBaselineSuffix, 0)
	if (overwrite==0 && exists(strNewName))
		DoAlert 1, strNewName + " exists. Overwrite?"
		if (V_flag == 2)
			return 0
		endif
	endif
	RemoveFromGraph /W=$graphStr /Z $strNewName
	Duplicate /o w_base CurrentDfr:$strNewName
	wave /Z newbase= $strNewName
	
	// save a copy of baseline-subtracted wave
	strNewName = CleanupName(NameOfWave(w) + ksSubtractedSuffix, 0)
	if (overwrite==0 && exists(strNewName))
		DoAlert 1, strNewName + " exists. Overwrite?"
		if (V_flag == 2)
			return 0
		endif
	endif
	RemoveFromGraph /W=$graphStr/Z $strNewName
	Duplicate /o w_sub CurrentDfr:$strNewName
	wave subtracted = $strNewName

	string newWaves = nameofwave(subtracted) + "," + nameofwave(newbase)
		
	if (WaveExists(w_dataX))		
		if (numpnts(w_baseX) < numpnts(w_dataX)) // using a subrange
			strNewName = CleanupName(NameOfWave(w_dataX),0)
			strNewName = UniqueName(strNewName, 1, 1)
			Duplicate /O w_baseX CurrentDfr:$strNewName
			wave w_x = $strNewName
			newWaves += "," + strNewName
		else
			wave w_x = w_dataX
		endif
		
		// update wavenotes to record the paired x wave
		string s_note = note(subtracted)
		s_note = ReplaceStringByKey("Xwave", s_note, NameOfWave(w_x), ":", "\r")
		note /K subtracted, s_note
		s_note = note(newbase)
		s_note = ReplaceStringByKey("Xwave", s_note, NameOfWave(w_x), ":", "\r")
		note /K newbase, s_note
	endif
	
//	note from w_sub was already transferred in duplicate operation
// append note from data wave to output wave note
	string strNote = note(w_base)
	sprintf strNote, "%sData=%s;Output=%s;", strNote, nameofwave(w), newWaves
	note subtracted, strNote
	print strNote
	
	if (kRecolorTraces == 1) // for all-at-once fitting
		restoreTraceColor(graphStr, traceStr)
	endif
	
	// add baseline-subtracted output wave to plot
	appendToSameAxes(graphStr, traceStr, subtracted, w_x, matchRGB=1, unique=1)
	
	// possibly plot baseline
	if (kAppendBaselines)
		appendToSameAxes(graphStr, traceStr, newbase, w_x, offset=1, matchRGB=1, unique=1)
	endif
	
	if (kRemoveOriginal)
		RemoveFromGraph /W=$graphStr/Z $traceStr, w_base
//		PopupMenu popTrace, win=$panelStr, mode=1
		// for All-At-Once fitting we don't want depth to be reset
//		resetDepth()		
//		updatePlot()		
	endif
end

static function clearGraph()
	
	string graphStr = getGraph()
	
	if (strlen(graphStr))
		RemoveFromGraph /W=$graphStr/Z w_base, w_sub, w_nodesY
		SetWindow $graphStr hook(hArcHull)=$""
		KillWindow $graphStr+"#CCH_panel"

		if (DataFolderRefStatus(dfr)>0 && kRecolorTraces==1)
			resetColors(graphStr, 0)
		endif
	endif
end

// for Igor 6 compatibility
static function /S getGraph()

	#if IgorVersion() >= 7
		GetWindow /Z CCH_panel  activeSW
		return ParseFilePath(0, s_value, "#", 0, 0)
	#endif
	
	string listStr = WinList("*", ";","WIN:1")
	string panelStr, graphStr
	variable i
	do
		graphStr = StringFromList(i, listStr)
		panelStr = graphStr + "#CCH_panel"
		if (WinType(panelStr))
			return graphStr
		endif
		i += 1
	while (i < ItemsInList(listStr))
	return ""
end

// Plots w, optionally vs w_x, on the same axes as the already plotted
// trace 'traceStr'
// appendToSameAxes(graphStr, traceStr, w, w_x, w_rgb={r,g,b}, offset=1)
// appends w (vs w_x if w_x exists), sets color to (r,g,b) and matches y
// offset of traceStr
// Default is choose a color that contrasts with that of traceStr; matchRGB=1 forces
// color to match
// fill = val adds fill to zero with opacity=val and sends trace to back
// unique = 1 removes any other instances of wave w from the graph
// replace = 1 removes tracestr after plotting w
static function appendToSameAxes(graphStr, traceStr, w, w_x, [w_rgb, offset, matchRGB, fill, lsize, unique, replace])
	string graphStr, traceStr
	wave /Z w, w_x, w_rgb
	variable offset  // match y offset
	int matchRGB // match color of already plotted trace
	variable fill // 10% opacity fill to zero
	variable lsize
	int unique, replace
	
	offset = ParamIsDefault(offset) ? 0 : offset
	matchRGB = ParamIsDefault(matchRGB) ? 0 : matchRGB
	fill = ParamIsDefault(fill) ? 0 : min(fill, 1)
	lsize = ParamIsDefault(lsize) ? 1 : lsize
	unique = ParamIsDefault(unique) ? 0 : unique
	replace = ParamIsDefault(replace) ? 0 : replace
	
	string s_info = TraceInfo(graphStr, traceStr, 0)
	string s_Xax = StringByKey("XAXIS",s_info)
	string s_Yax = StringByKey("YAXIS",s_info)
	string s_flags = StringByKey("AXISFLAGS",s_info)
	variable flagBits = GrepString(s_flags, "/R") + 2*GrepString(s_flags, "/T")
	offset = offset ? GetOffsetFromInfoString(s_info, 1) : 0
	
	// get color of already plotted trace
	variable c0, c1, c2
	sscanf ListMatch(s_info, "rgb(x)=*"), "rgb(x)=(%d,%d,%d", c0, c1, c2
	
	if (matchRGB==0 && ParamIsDefault(w_rgb)) // no color specified
		STRUCT RGBcolor RGB
		RGB.red = c0; RGB.green = c1; RGB.blue = c2
		contrastingColor(RGB)
		Make /free w_RGB = {RGB.red, RGB.green, RGB.blue}
	endif
	if (matchRGB) // this overides any specified color
		Make /free/O w_RGB = {c0,c1,c2}
	endif
	
	int i, numTraces
	string traces, trace
	if (unique)
		traces = TraceNameList(graphStr,";",1)
		for (i=ItemsInList(traces)-1;i>=0;i-=1)
			trace = StringFromList(i,traces)
			wave /Z ithTraceWave = TraceNameToWaveRef(graphStr, trace)
			if (WaveRefsEqual(ithTraceWave, w))
				RemoveFromGraph /W=$graphStr/Z $trace
			endif
		endfor
	endif
	
	switch (flagBits)
		case 0:
			if (WaveExists(w_x))
				AppendToGraph /W=$graphStr/B=$s_Xax/L=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w vs w_x
			elseif (DimSize(w,1)==2)
				AppendToGraph /W=$graphStr/B=$s_Xax/L=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w[][1] vs w[][0]
			else
				AppendToGraph /W=$graphStr/B=$s_Xax/L=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w
			endif
			break
		case 1:
			if (WaveExists(w_x))
				AppendToGraph /W=$graphStr/B=$s_Xax/R=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w vs w_x
			elseif (DimSize(w,1)==2)
				AppendToGraph /W=$graphStr/B=$s_Xax/R=$s_Yax /C=(w_rgb[0],w_rgb[1],w_rgb[2]) w[][1] vs w[][0]
			else
				AppendToGraph /W=$graphStr/B=$s_Xax/R=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w
			endif
			break
		case 2:
			if (WaveExists(w_x))
				AppendToGraph /W=$graphStr/T=$s_Xax/L=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w vs w_x
			elseif (DimSize(w,1)==2)
				AppendToGraph /W=$graphStr/T=$s_Xax/L=$s_Yax /C=(w_rgb[0],w_rgb[1],w_rgb[2]) w[][1] vs w[][0]
			else
				AppendToGraph /W=$graphStr/T=$s_Xax/L=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w
			endif
			break
		case 3:
			if (WaveExists(w_x))
				AppendToGraph /W=$graphStr/T=$s_Xax/R=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w vs w_x
			elseif (DimSize(w,1)==2)
				AppendToGraph /W=$graphStr/T=$s_Xax/R=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w[][1] vs w[][0]
			else
				AppendToGraph /W=$graphStr/T=$s_Xax/R=$s_Yax/C=(w_rgb[0],w_rgb[1],w_rgb[2]) w
			endif
			break
	endswitch
	ModifyGraph /W=$graphStr offset($NameOfWave(w))={0,offset}, lsize($NameOfWave(w))=lsize
	
	if (fill > 0)
		ModifyGraph /W=$graphStr mode($NameOfWave(w))=7, hbFill($NameOfWave(w))=2
		ModifyGraph /W=$graphStr usePlusRGB($NameOfWave(w))=1, plusRGB($NameOfWave(w))=(w_rgb[0],w_rgb[1],w_rgb[2],fill*65535)
		ModifyGraph /W=$graphStr useNegRGB($NameOfWave(w))=1, negRGB($NameOfWave(w))=(w_rgb[0],w_rgb[1],w_rgb[2],fill*65535)
		ReorderTraces /W=$graphStr _back_, {$NameOfWave(w)}
	endif
	
	if (replace)
		RemoveFromGraph /W=$graphStr traceStr
	endif
end

static function contrastingColor(STRUCT RGBcolor &RGB)
	STRUCT HSLcolor HSL
	RGB2HSL(RGB, HSL)
	if (HSL.S<0.3)
		RGB.red = 65535;RGB.green = 0;RGB.blue = 0
		return 1
	endif
	variable hue = round(6*HSL.H)
	hue -= 6*(hue>5)
	switch (hue)
		case 0: // red
			RGB.red = 0; RGB.green = 65535; RGB.blue = 65535 // cyan
			break
		case 1: // yellow
			RGB.red = 0; RGB.green = 0; RGB.blue = 65535 // blue
			break
		case 2: // green
			RGB.red = 65535; RGB.green = 0; RGB.blue = 65535 // magenta
			break
		case 3: // cyan
			RGB.red = 65535; RGB.green = 0; RGB.blue = 0 // red
			break
		case 4: // blue
//			RGB.red = 65535; RGB.green = 65535; RGB.blue = 0 // yellow
			RGB.red = 65535; RGB.green = 43690; RGB.blue = 0 // orange, yellow is hard to see
			break
		case 5: // magenta
			RGB.red = 0; RGB.green = 65535; RGB.blue = 0 // lime
			break
	endswitch
	return 1
end

// see easyrgb.com
static function RGB2HSL(STRUCT RGBcolor &rgb, STRUCT HSLcolor &hsl)
	// R, G and B input range = 0 - 65535
	// H, S and L output range = 0 - 1
	variable red = rgb.red/65535
	variable green = rgb.green/65535
	variable blue = rgb.blue/65535

	variable var_Min = min(red,green,blue)
	variable var_Max = max(red,green,blue)
	variable del_Max = var_Max - var_Min
	
	variable H,S,L
	L = (var_Max + var_Min)/ 2
	if (del_Max == 0) // grey
	    H = 0
	    S = 0
	else // Chromatic data
		S = (L < 0.5) ? del_Max/(var_Max + var_Min) : del_Max/(2 - var_Max - var_Min)
		
		variable del_R = ( (var_Max - red)/6 + del_Max/2 ) / del_Max
		variable del_G = ( (var_Max - green)/6 + del_Max/2 ) / del_Max
		variable del_B = ( (var_Max - blue)/6 + del_Max/2 ) / del_Max
		
		if (red == var_Max )
			H = del_B - del_G
		elseif (green == var_Max )
			H = (1/3) + del_R - del_B
		elseif (blue == var_Max )
			H = (2/3) + del_G - del_R
		endif
		H += (H < 0)
		H -= (H > 1)
	endif
	hsl.H = H
	hsl.S = S
	hsl.L = L
	return 1
end

static structure HSLcolor
	float H
	float S
	float L
endstructure

static function GetOffsetFromInfoString(string s, int axis)
	variable xOffset, yOffset
	sscanf ListMatch(s, "offset(x)=*"), "offset(x)={%g,%g}", xOffset, yOffset
	return axis ? yOffset : xOffset
end

static function /WAVE getColor(graphStr, traceStr)
	string graphStr, traceStr
		
	variable c0, c1, c2
	string infoStr = TraceInfo(graphStr, traceStr, 0)
	variable startIndex = strsearch(infoStr, ";rgb(x)=", 0)
	if (startIndex == -1)
		return $"{}"
	endif
	sscanf infoStr[startIndex,strlen(infoStr)-1], ";rgb(x)=(%d,%d,%d*", c0, c1, c2
	Make /free w_rgb={c0,c1,c2}
	return w_rgb
end


// Shen et al (2018) Automatic baseline correction method for the
// open-path Fourier transform infrared spectra by using simple iterative
// averaging. Optics Express 26, A609-A614.
// https:doi.org/10.1364/OE.26.00A609
// This is slow, so I haven't incorporated it into the archull panel.
static function IterativeAveraging(w)
	wave w
	Duplicate /O w w_bl
	Duplicate /free w, w_previous, w_diff
	variable Pmax = numpnts(w)-1, i = 0
	variable Pstart, Pend, Sabs, previousSabs, deltaSabs
	
	variable threshold = 0.005, maxits = 1e3
	variable timeout = datetime + 3
	do
		previousSabs = Sabs
		w_previous = w_bl
		Pstart = 1
		Pend = Pmax - 1
		do
			w_bl[Pstart,Pend] = min(w_bl, (w_bl[p-1]+w_bl[p+1])/2)
			Pstart += 1
			Pend -= 1
		while(Pstart < Pmax/2)
		
		w_diff = abs(w_bl-w_previous)
		Sabs = sum(w_diff)
		deltaSabs = abs(Sabs-previousSabs)
		i += 1
		if (datetime > timeout)
			Print "baseline calculation timed out"
			break
		endif
	while((deltaSabs/Sabs)>threshold && i<maxits)
	Print i, deltaSabs/Sabs
end

static function makeSpectrum()	
	string strName = UniqueName("foo", 1, 0)
	Make /N=1000 $(UniqueName("foo", 1, 0)) /WAVE=foo
	variable a = enoise(5), b = enoise(1e-3), c = enoise(1e-5), d = enoise(400)
	foo=gnoise(0.1)+a+b*x+c*(x-d)^2
	variable i, height, position, width
	for(i=0;i<5;i+=1)
		height=150+enoise(50)
		position=500+enoise(400)
		width=20+enoise(10)
		foo+=height*Gauss(x, position, width/sqrt(2))
	endfor
	Display /K=1 foo
end

// execute archull#demo() to create a fake spectrum for demo
static function demo()
	makespectrum()
	Initialise()
end

static function goodbye()
	
	if (IgorVersion() >= 8)			
		
		string msg = "All of the baseline types developed in the ArcHull package are now included in the Baselines project.\r\r"

		if (strlen(functionInfo("baselines#DoSplineFit")))
			msg += "Please uninstall ArcHull and instead use the Baselines utility:\r\r"
			msg += "                   Analysis -> Baselines\r\rThanks!"
			doAlert 0, msg
			return 1
		endif

		if (strlen(functionInfo("updater#update")))
			msg += "Do you want to open IgorExchange project installer to look for the most recent version of Baselines?"
			DoAlert 1, msg
			if (v_flag == 1)
				execute "updater#makeInstallerPanel()"
				return 1
			endif
			return 0	
		endif
		
		msg += "Please uninstall ArcHull and install the most recent version (≥ 4.0) of the Baselines package.\r\r"
		msg += "Do you want to visit https://www.wavemetrics.com/project/Baselines?"
		
		DoAlert 1, msg
		if (v_flag == 1)
			BrowseURL /Z "https://www.wavemetrics.com/project/Baselines"
			return 1
		endif		
	endif
	
	return 0
end